
#include <stdlib.h>
#include <math.h>

// May need to change this include to customize to your environment
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "st.h"
#include "response.h"

using namespace std;

enum MODE {
	MODE_RESPONSE,
	MODE_CREATE,
	MODE_VIEW,
	MODE_VP,
	MODE_TONEMAP
};

// delta used to make sure we don't do log(0)
#define LOG_DELTA .000001f

// Current size of the OpenGL window
int windowSizeX;
int windowSizeY;

STScene scene;				// The scene graph
STTexture* square_tex;
STSceneNode* square_node;	// The node containing the square
STImage image;

CameraResponse cr;			// Camera response, either generated or loaded
STHDRImage hdr;				// The high dynamic range image

int mode;					// The type of operation we're doing

float shutter_time = .125f; // Shutter time for virtual photograph
float key_lum = .45f;		// The key value for tonemapping
float hdr_view_max = 1.f;	// Value to map to max color

/* Recover an HDR image from a set of photos and the response curve for the camera used to
 * take the photos. To work efficiently you should only have one image open at a time.
 * For each pixel, you want to calculate the following:
 *
 * Ls(x,y) = sum_i( weight( pixel(x,y,i) ) * ( Exposure( pixel(x,y,i) ) - ln(dt_i) );
 * ws(x,y) = sum_i( weight( pixel(x,y,i) ) );
 * L_w(x,y) = exp( Ls(x,y) / ws(x,y) ).
 */
void recover_hdr(vector<Photo>& photos, CameraResponse& response, STHDRImage& hdr_image) {

}

/* Scale an HDR image for viewing, this is just a linear map
 * of hdr such that 0 maps to 0 and max_val maps to 255.
 */
void scale_hdr(STHDRImage& hdr, float max_val, STImage& result) {

}

/* Take a virtual photo - Use the camera response curve to take a virtual photo using
 * the given shutter time.  Use hdr as the source image and store the virtual photo
 * in result.  Remember that a photo is just the response of the camera to the incoming
 * radiance over a given shutter time.
 */
void virtual_photo(STHDRImage& hdr, CameraResponse& response, float shutter, STImage& result) {

}

/* Tonemap operator, tonemaps the hdr image and stores the
 * tonemapped image in result.  Use the value key as the 
 * key of the photograph.  This function should:
 * 1) Resize the result image to the right size
 * 2) Calculate the log-average luminance, Lavg
 * 3) Calculate the tonemapped image using the equation
 *    L(x,y) = L_w(x,y) * key / Lavg;
 *    C_tm(x,y) = L(x,y) / (1 + L(x,y));
 * 4) Store those tonemapped values to the result image,
 *    scaling appropriately.
 */
void tonemap(STHDRImage& hdr, float key, STImage& result) {

}

static void setup() {
	image.Resize(1,1);

	STSceneNode* root = scene.GetRoot();
	STShape* square = STShape::CreateRect(1.f, 1.f);
	square->SetColor(STColor(1.f, 1.f, 1.f));

	square_tex = new STTexture();
	square_tex->Upload(image, false);
	square->SetTexture(square_tex);
	
	square_node = root->PlaceObj(square);
	square_node->SetTransform( STTransform::Translate(.5f, .5f, 0.f) );
}

static void display() {
	glClearColor(.3f, .3f, .3f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	scene.Draw();

	glutSwapBuffers();
}

// Keyboard event handler:
static void keyboard(unsigned char key, int x, int y) {
	switch (key) {
		// quit
		case '+':
		case '=':
			if (mode == MODE_VIEW) {
				hdr_view_max /= 2.f;
				scale_hdr(hdr, hdr_view_max, image);
				square_tex->Upload(image, false);
			}
			else if (mode == MODE_VP) {
				shutter_time *= powf(2.f, 1.f/3.f); // 1/3 stop
				virtual_photo(hdr, cr, shutter_time, image);
				square_tex->Upload(image, false);
			}
			else if (mode == MODE_TONEMAP) {
				key_lum += .05f;
				tonemap(hdr, key_lum, image);
				square_tex->Upload(image, false);
			}
			break;
		case '-':
		case '_':
			if (mode == MODE_VIEW) {
				hdr_view_max *= 2.f;
				scale_hdr(hdr, hdr_view_max, image);
				square_tex->Upload(image, false);
			}
			else if (mode == MODE_VP) {
				shutter_time /= powf(2.f, 1.f/3.f); // 1/3 stop
				virtual_photo(hdr, cr, shutter_time, image);
				square_tex->Upload(image, false);
			}
			else if (mode == MODE_TONEMAP) {
				key_lum -= .05f;
				tonemap(hdr, key_lum, image);
				square_tex->Upload(image, false);
			}

			break;
		case 's':
		case 'S':
			if (mode == MODE_VIEW)
				image.Save("view.jpg");
			else if (mode == MODE_VP)
				image.Save("vp.jpg");
			else if (mode == MODE_TONEMAP)
				image.Save("tonemap.jpg");
			break;
		case char(27):
		case 'q':
		case 'Q':
			exit(0);
	}

	glutPostRedisplay();
}

void reshape(int w, int h) {
	windowSizeX = w;
	windowSizeY = h;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	gluOrtho2D(0., 1., 0., 1.);
	glViewport(0, 0, w, h);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glutPostRedisplay();
}

static void usage() {
	printf("Usage:\n");
	printf("hdr -response photos.list response_out.cr\n");
	printf("hdr -create photos.list response.cr photo_out.pfm\n");
	printf("hdr -view photo.pfm\n");
	printf("hdr -vp photo.pfm response.cr\n");
	printf("hdr -tonemap photo.pfm\n");
	exit(-1);
}

int main(int argc, char** argv) {

	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowPosition(20,20);
	glutInitWindowSize(512, 512);
	glutCreateWindow("CS148 - Assignment 5");
	
	setup();

	if (argc < 2) {
		usage();
	}

	if ( strcmp(argv[1], "-response") == 0) {
		if (argc != 4)
			usage();
		mode = MODE_RESPONSE;
		
		// Defaults
		float lambda = 50.f;
		int nsamples = 100;

		vector<Photo> photos;
		LoadHDRStack(argv[2], photos);
		cr.SolveForResponse(photos, lambda, nsamples);

		cr.Save(argv[3]);
		exit(0);
	}
	else if ( strcmp(argv[1], "-create") == 0) {
		if (argc != 5)
			usage();
		mode = MODE_CREATE;
		vector<Photo> photos;
		LoadHDRStack(argv[2], photos);
		cr.Load(argv[3]);
		recover_hdr(photos, cr, hdr);
		hdr.Save(argv[4]);
		exit(0);
	}
	else if ( strcmp(argv[1], "-view") == 0) {
		if (argc != 3)
			usage();
		mode = MODE_VIEW;
		hdr.Load(argv[2]);
		scale_hdr(hdr, hdr_view_max, image);
		square_tex->Upload(image, false);
	}
	else if ( strcmp(argv[1], "-vp") == 0) {
		if (argc != 4)
			usage();
		mode = MODE_VP;
		hdr.Load(argv[2]);
		cr.Load(argv[3]);
		virtual_photo(hdr, cr, shutter_time, image);
		square_tex->Upload(image, false);
	}
	else if ( strcmp(argv[1], "-tonemap") == 0) {
		if (argc != 3)
			usage();
		mode = MODE_TONEMAP;
		hdr.Load(argv[2]);
		tonemap(hdr, key_lum, image);
		square_tex->Upload(image, false);
	}

	glutReshapeWindow(image.GetWidth(), image.GetHeight());

	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutReshapeFunc(reshape);
	glutMainLoop();

	return 0;
}
