
#ifndef __STSHAPE_H__
#define __STSHAPE_H__

#include <vector>

#include "st.h"


/**
*  Simple class to represent 2D points
*/
class STPoint2D {
	public:
	float x, y;

	STPoint2D() { }

	STPoint2D(float px, float py)
		: x(px), y(py) { } 	
};


/**
* STShapes are 2D shapes.  The internal format of a shape is a list of
* vertices paired with a list of triangles each defined by a set of 3 vertex
* indices.  STShape loads shape data from text files that are very similar
* to OBJ files in format.  There is support for per-vertex texture coordinates.
*
* A few static convenience methods exist that create shapes without requiring
* file IO.  Eg. creating circles and boxes.
*/
class STShape : public STDrawable {

	public:

		STShape();
		~STShape();

		void     Draw();

		STStatus Load(const char* filename);
		const STColor&  GetColor() const;
		void	 SetColor( const STColor& c);
		STTexture* GetTexture() const;
		void SetTexture(STTexture* _tex);

		static STShape*  CreateCircle(float r);
		static STShape*  CreateRect(float w, float h);

	private:
			
		STColor				   mColor;
		std::vector<STPoint2D> mVerts;
		std::vector<STPoint2D> mTexCoords;
		std::vector<int>       mFaceVertIndices;
		std::vector<int>       mFaceTexIndices;
		STTexture*             mTex;

};



#endif  // __STSHAPE_H__


